@@ -3,13 +3,13 @@
 """Server module."""
 import os
 import sys
+import importlib.metadata
+from urllib.parse import urlsplit, urlunsplit, parse_qs
 from typing import Union  # NOQA; type: ignore
-from urllib.parse import urlparse
 
+from flask import Flask
 from flask.cli import FlaskGroup
 from flask_admin import Admin
-from flask_api import FlaskAPI
-from flask_bootstrap import Bootstrap
 
 import buku
 from buku import BukuDb, __version__
@@ -20,21 +20,22 @@ except ImportError:
     from bukuserver.middleware import ReverseProxyPrefixFix
 import click
 import flask
-from flask import __version__ as flask_version  # type: ignore
 from flask import current_app, redirect, request, url_for
 
+flask_version = importlib.metadata.version('flask')
+
 try:
-    from . import api, views
+    from . import api, views, util, _p, _l, gettext, ngettext
     from response import Response
 except ImportError:
-    from bukuserver import api, views
+    from bukuserver import api, views, util, _p, _l, gettext, ngettext
     from bukuserver.response import Response
 
 
 STATISTIC_DATA = None
 
 def _fetch_data():
-    url = request.data.get('url')
+    url = request.form.get('url')
     try:
         return (None if not url else buku.fetch_data(url))
     except Exception as e:
@@ -57,18 +58,16 @@ def refresh_bookmark(rec_id: Union[int, None]):
     return Response.from_flag(result_flag)
 
 
-def get_tiny_url(rec_id):
-    url = getattr(flask.g, 'bukudb', api.get_bukudb()).tnyfy_url(rec_id)
-    return Response.SUCCESS(data={'url': url}) if url else Response.FAILURE()
+get_tiny_url = lambda rec_id: Response.REMOVED()
 
 
 _BOOL_VALUES = {'true': True, '1': True, 'false': False, '0': False}
-def get_bool_from_env_var(key: str, default_value: bool) -> bool:
+def get_bool_from_env_var(key: str, default_value: bool = False) -> bool:
     """Get bool value from env var."""
     return _BOOL_VALUES.get(os.getenv(key, '').lower(), default_value)
 
 
-def init_locale(app):
+def init_locale(app, context_processor=lambda: {}):
     try:  # as per Flask-Admin-1.6.1
         try:
             from flask_babelex import Babel
@@ -76,28 +75,52 @@ def init_locale(app):
         except ImportError:
             from flask_babel import Babel
             Babel().init_app(app, locale_selector=lambda: app.config['BUKUSERVER_LOCALE'])
-    except Exception:
-        app.logger.warning('failed to init locale')
+        app.context_processor(lambda: {'lang': app.config['BUKUSERVER_LOCALE'] or 'en', **context_processor()})
+    except Exception as e:
+        app.jinja_env.add_extension('jinja2.ext.i18n')
+        app.jinja_env.install_gettext_callables(gettext, ngettext, newstyle=True)
+        app.logger.warning(f'failed to init locale ({e})')
+        app.context_processor(lambda: {'lang': '', **context_processor()})
+
+
+# handling popup= URL argument
+def before_request():
+    _post_popup = request.headers.get('Content-Type') != 'application/json' and request.form.get('popup')
+    flask.g.popup = request.args.get('popup') or _post_popup
+
+# applying popup= to the redirect URL
+def after_request(response):
+    if flask.g.popup and 'Location' in response.headers:
+        _scheme, _netloc, _path, _query, _fragment = urlsplit(response.headers['Location'])
+        _params = parse_qs(_query)
+        if not _params.get('popup'):
+            _query = '&'.join(s for s in [_query, 'popup=True'] if s)
+            response.headers['Location'] = urlunsplit((_scheme, _netloc, _path, _query, _fragment))
+    return response
 
 
 def create_app(db_file=None):
     """create app."""
-    app = FlaskAPI(__name__)
+    app = Flask(__name__)
+    db_file = os.getenv('BUKUSERVER_DB_FILE') or db_file
+    if db_file and not os.path.dirname(db_file) and not os.path.splitext(db_file)[1]:
+        db_file = os.path.join(BukuDb.get_default_dbdir(), db_file + '.db')
+    os.environ.setdefault('FLASK_DEBUG', ('1' if get_bool_from_env_var('BUKUSERVER_DEBUG') else '0'))
     per_page = int(os.getenv('BUKUSERVER_PER_PAGE', str(views.DEFAULT_PER_PAGE)))
     per_page = per_page if per_page > 0 else views.DEFAULT_PER_PAGE
     app.config['BUKUSERVER_PER_PAGE'] = per_page
     url_render_mode = os.getenv('BUKUSERVER_URL_RENDER_MODE', views.DEFAULT_URL_RENDER_MODE)
-    if url_render_mode not in ('full', 'netloc'):
+    if url_render_mode not in ('full', 'netloc', 'netloc-tag'):
         url_render_mode = views.DEFAULT_URL_RENDER_MODE
     app.config['BUKUSERVER_URL_RENDER_MODE'] = url_render_mode
     app.config['SECRET_KEY'] = os.getenv('BUKUSERVER_SECRET_KEY') or os.urandom(24)
     app.config['BUKUSERVER_READONLY'] = \
-        get_bool_from_env_var('BUKUSERVER_READONLY', False)
+        get_bool_from_env_var('BUKUSERVER_READONLY')
     app.config['BUKUSERVER_DISABLE_FAVICON'] = \
         get_bool_from_env_var('BUKUSERVER_DISABLE_FAVICON', True)
     app.config['BUKUSERVER_OPEN_IN_NEW_TAB'] = \
-        get_bool_from_env_var('BUKUSERVER_OPEN_IN_NEW_TAB', False)
-    app.config['BUKUSERVER_DB_FILE'] = os.getenv('BUKUSERVER_DB_FILE') or db_file
+        get_bool_from_env_var('BUKUSERVER_OPEN_IN_NEW_TAB')
+    app.config['BUKUSERVER_DB_FILE'] = db_file
     reverse_proxy_path = os.getenv('BUKUSERVER_REVERSE_PROXY_PATH')
     if reverse_proxy_path:
         if not reverse_proxy_path.startswith('/'):
@@ -106,21 +129,23 @@ def create_app(db_file=None):
             print('Warning: reverse proxy path should not include trailing slash')
         app.config['REVERSE_PROXY_PATH'] = reverse_proxy_path
         ReverseProxyPrefixFix(app)
-    bukudb = BukuDb(dbfile=app.config['BUKUSERVER_DB_FILE'])
+    bukudb = BukuDb(dbfile=db_file)
     app.config['FLASK_ADMIN_SWATCH'] = (os.getenv('BUKUSERVER_THEME') or 'default').lower()
     app.config['BUKUSERVER_LOCALE'] = os.getenv('BUKUSERVER_LOCALE') or 'en'
     app.app_context().push()
     setattr(flask.g, 'bukudb', bukudb)
     init_locale(app)
+    app.before_request(before_request)
+    app.after_request(after_request)
 
     @app.shell_context_processor
     def shell_context():
         """Shell context definition."""
         return {'app': app, 'bukudb': bukudb}
 
-    app.jinja_env.filters['netloc'] = lambda x: urlparse(x).netloc  # pylint: disable=no-member
+    app.jinja_env.filters.update(util.JINJA_FILTERS)
+    app.jinja_env.globals.update(_p=_p, dbfile=bukudb.dbfile, dbname=bukudb.dbname)
 
-    Bootstrap(app)
     admin = Admin(
         app, name='buku server', template_mode='bootstrap3',
         index_view=views.CustomAdminIndexView(
@@ -154,9 +179,9 @@ def create_app(db_file=None):
     def favicon():
         return redirect(url_for('static', filename='bukuserver/favicon.svg'), code=301)  # permanent redirect
 
-    admin.add_view(views.BookmarkModelView(bukudb, 'Bookmarks'))
-    admin.add_view(views.TagModelView(bukudb, 'Tags'))
-    admin.add_view(views.StatisticView(bukudb, 'Statistic', endpoint='statistic'))
+    admin.add_view(views.BookmarkModelView(bukudb, _l('Bookmarks')))
+    admin.add_view(views.TagModelView(bukudb, _l('Tags')))
+    admin.add_view(views.StatisticView(bukudb, _l('Statistic'), endpoint='statistic'))
     return app
 
 
